@@ -1,7 +1,6 @@
 ;;; calc-help.el --- help display functions for Calc,
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007, 2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2012  Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
@@ -128,7 +127,7 @@ C-w  Describe how there is no warranty for Calc."
               (dig2 (char-after (match-beginning 3))))
           (delete-region (match-end 1) (match-end 0))
           (goto-char (match-beginning 1))
-          (delete-backward-char 1)
+          (delete-char -1)
           (delete-char 5)
           (insert (format "%c .. %c" (min dig1 dig2) (max dig1 dig2)))))
       (goto-char (point-min)))))
@@ -446,6 +445,7 @@ C-w  Describe how there is no warranty for Calc."
 	    '(calc-inverse-prefix-help
 	      calc-hyperbolic-prefix-help
 	      calc-inv-hyp-prefix-help
+              calc-option-prefix-help
 	      calc-a-prefix-help
 	      calc-b-prefix-help
 	      calc-c-prefix-help
@@ -455,6 +455,7 @@ C-w  Describe how there is no warranty for Calc."
 	      calc-h-prefix-help
 	      calc-j-prefix-help
 	      calc-k-prefix-help
+	      calc-l-prefix-help
 	      calc-m-prefix-help
 	      calc-r-prefix-help
 	      calc-s-prefix-help
@@ -512,6 +513,11 @@ C-w  Describe how there is no warranty for Calc."
      "I H + a S (general invert func); v h (rtail)")
    "inverse-hyperbolic" nil))
 
+(defun calc-option-prefix-help ()
+  (interactive)
+  (calc-do-prefix-help
+   '("")
+   "option" nil))
 
 (defun calc-f-prefix-help ()
   (interactive)
@@ -663,13 +669,21 @@ C-w  Describe how there is no warranty for Calc."
      "SHIFT + stat: + (sum), - (asum), * (prod), # (count)")
    "units/stat" ?u))
 
+(defun calc-l-prefix-help ()
+  (interactive)
+  (calc-do-prefix-help
+   '("Quantity, DB level, Np level"
+     "+, -, *, /"
+     "Scientific pitch notation, Midi number, Frequency"
+     )
+   "log units" ?l))
 
 (defun calc-v-prefix-help ()
   (interactive)
   (calc-do-prefix-help
    '("Pack, Unpack, Identity, Diagonal, indeX, Build"
      "Row, Column, Subvector; Length; Find; Mask, Expand"
-     "Tranpose, Arrange, reVerse; Head, Kons; rNorm"
+     "Transpose, Arrange, reVerse; Head, Kons; rNorm"
      "SHIFT + Det, & (inverse), LUD, Trace, conJtrn, Cross"
      "SHIFT + Sort, Grade, Histogram; cNorm"
      "SHIFT + Apply, Map, Reduce, accUm, Inner-, Outer-prod"
@@ -682,5 +696,4 @@ C-w  Describe how there is no warranty for Calc."
 
 (provide 'calc-help)
 
-;; arch-tag: 2d347593-7591-449e-a64a-93dab5f2f686
 ;;; calc-help.el ends here
