@@ -1,17 +1,16 @@
 ;;; calc-prog.el --- user programmability functions for Calc
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2012 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -32,6 +29,11 @@
 (require 'calc-ext)
 (require 'calc-macs)
 
+;; Declare functions which are defined elsewhere.
+(declare-function edmacro-format-keys "edmacro" (macro &optional verbose))
+(declare-function edmacro-parse-keys "edmacro" (string &optional need-vector))
+(declare-function math-read-expr-level "calc-aent" (exp-prec &optional exp-term))
+
 
 (defun calc-equal-to (arg)
   (interactive "P")
@@ -160,7 +162,7 @@
 (defvar math-integral-cache-state)
 
 ;; calc-user-formula-alist is local to calc-user-define-formula,
-;; calc-user-define-compostion and calc-finish-formula-edit,
+;; calc-user-define-composition and calc-finish-formula-edit,
 ;; but is used by calc-fix-user-formula.
 (defvar calc-user-formula-alist)
 
@@ -168,17 +170,17 @@
   (interactive)
   (calc-wrapper
    (let* ((form (calc-top 1))
-	  (arglist nil)
+	  (math-arglist nil)
 	  (is-lambda (and (eq (car-safe form) 'calcFunc-lambda)
 			  (>= (length form) 2)))
 	  odef key keyname cmd cmd-base cmd-base-default
           func calc-user-formula-alist is-symb)
      (if is-lambda
-	 (setq arglist (mapcar (function (lambda (x) (nth 1 x)))
+	 (setq math-arglist (mapcar (function (lambda (x) (nth 1 x)))
 			       (nreverse (cdr (reverse (cdr form)))))
 	       form (nth (1- (length form)) form))
        (calc-default-formula-arglist form)
-       (setq arglist (sort arglist 'string-lessp)))
+       (setq math-arglist (sort math-arglist 'string-lessp)))
      (message "Define user key: z-")
      (setq key (read-char))
      (if (= (calc-user-function-classify key) 0)
@@ -196,7 +198,7 @@
      (while
 	 (progn
 	   (setq cmd-base-default (concat "User-" keyname))
-           (setq cmd (completing-read 
+           (setq cmd (completing-read
                       (concat "Define M-x command name (default calc-"
                               cmd-base-default
                               "): ")
@@ -221,7 +223,7 @@
 		    "That name conflicts with a built-in Emacs function.  Replace this function? "))))))
      (while
 	 (progn
-           (setq cmd-base-default     
+           (setq cmd-base-default
                  (if cmd-base
                      (if (string-match
                           "\\`User-.+" cmd-base)
@@ -230,16 +232,16 @@
                           (substring cmd-base 5))
                        cmd-base)
                    (concat "User" keyname)))
-	   (setq func 
+	   (setq func
                  (concat "calcFunc-"
-                         (completing-read 
+                         (completing-read
                           (concat "Define algebraic function name (default "
                                   cmd-base-default "): ")
                           (mapcar (lambda (x) (substring x 9))
                                   (all-completions "calcFunc-"
                                                    obarray))
-                          (lambda (x) 
-                            (fboundp 
+                          (lambda (x)
+                            (fboundp
                              (intern (concat "calcFunc-" x))))
                           nil)))
            (setq func
@@ -264,24 +266,24 @@
 					(format "%05d" (% (random) 10000)))))))
 
      (if is-lambda
-	 (setq calc-user-formula-alist arglist)
+	 (setq calc-user-formula-alist math-arglist)
        (while
 	   (progn
-	     (setq calc-user-formula-alist 
+	     (setq calc-user-formula-alist
                    (read-from-minibuffer "Function argument list: "
-                                         (if arglist
-                                             (prin1-to-string arglist)
+                                         (if math-arglist
+                                             (prin1-to-string math-arglist)
                                            "()")
                                          minibuffer-local-map
                                          t))
-	     (and (not (calc-subsetp calc-user-formula-alist arglist))
+	     (and (not (calc-subsetp calc-user-formula-alist math-arglist))
 		  (not (y-or-n-p
 			"Okay for arguments that don't appear in formula to be ignored? "))))))
      (setq is-symb (and calc-user-formula-alist
 			func
 			(y-or-n-p
 			 "Leave it symbolic for non-constant arguments? ")))
-     (setq calc-user-formula-alist 
+     (setq calc-user-formula-alist
            (mapcar (function (lambda (x)
                                (or (cdr (assq x '((nil . arg-nil)
                                                   (t . arg-t))))
@@ -325,13 +327,14 @@
 	     (setcdr kmap (cons (cons key cmd) (cdr kmap)))))))
    (message "")))
 
+(defvar math-arglist)		    ; dynamically bound in all callers
 (defun calc-default-formula-arglist (form)
   (if (consp form)
       (if (eq (car form) 'var)
-	  (if (or (memq (nth 1 form) arglist)
+	  (if (or (memq (nth 1 form) math-arglist)
 		  (math-const-var form))
 	      ()
-	    (setq arglist (cons (nth 1 form) arglist)))
+	    (setq math-arglist (cons (nth 1 form) math-arglist)))
 	(calc-default-formula-arglist-step (cdr form)))))
 
 (defun calc-default-formula-arglist-step (l)
@@ -379,34 +382,34 @@
    (if (eq calc-language 'unform)
        (error "Can't define formats for unformatted mode"))
    (let* ((comp (calc-top 1))
-	  (func (intern 
+	  (func (intern
                  (concat "calcFunc-"
                          (completing-read "Define format for which function: "
                                           (mapcar (lambda (x) (substring x 9))
                                                   (all-completions "calcFunc-"
                                                                    obarray))
-                                          (lambda (x) 
-                                            (fboundp 
+                                          (lambda (x)
+                                            (fboundp
                                              (intern (concat "calcFunc-" x))))))))
 	  (comps (get func 'math-compose-forms))
 	  entry entry2
-	  (arglist nil)
+	  (math-arglist nil)
 	  (calc-user-formula-alist nil))
      (if (math-zerop comp)
 	 (if (setq entry (assq calc-language comps))
 	     (put func 'math-compose-forms (delq entry comps)))
        (calc-default-formula-arglist comp)
-       (setq arglist (sort arglist 'string-lessp))
+       (setq math-arglist (sort math-arglist 'string-lessp))
        (while
 	   (progn
-	     (setq calc-user-formula-alist 
+	     (setq calc-user-formula-alist
                    (read-from-minibuffer "Composition argument list: "
-                                         (if arglist
-                                             (prin1-to-string arglist)
+                                         (if math-arglist
+                                             (prin1-to-string math-arglist)
                                            "()")
                                          minibuffer-local-map
                                          t))
-	     (and (not (calc-subsetp calc-user-formula-alist arglist))
+	     (and (not (calc-subsetp calc-user-formula-alist math-arglist))
 		  (y-or-n-p
 		   "Okay for arguments that don't appear in formula to be invisible? "))))
        (or (setq entry (assq calc-language comps))
@@ -414,9 +417,9 @@
 		(cons (setq entry (list calc-language)) comps)))
        (or (setq entry2 (assq (length calc-user-formula-alist) (cdr entry)))
 	   (setcdr entry
-		   (cons (setq entry2 
+		   (cons (setq entry2
                                (list (length calc-user-formula-alist))) (cdr entry))))
-       (setcdr entry2 
+       (setcdr entry2
                (list 'lambda calc-user-formula-alist (calc-fix-user-formula comp))))
      (calc-pop-stack 1)
      (calc-do-refresh))))
@@ -500,8 +503,8 @@
   (switch-to-buffer calc-original-buffer))
 
 ;; The variable calc-lang is local to calc-write-parse-table, but is
-;; used by calc-write-parse-table-part which is called by 
-;; calc-write-parse-table.  The variable is also local to 
+;; used by calc-write-parse-table-part which is called by
+;; calc-write-parse-table.  The variable is also local to
 ;; calc-read-parse-table, but is used by calc-fix-token-name which
 ;; is called (indirectly) by calc-read-parse-table.
 (defvar calc-lang)
@@ -564,11 +567,10 @@
 	  (let ((pos (point)))
 	    (end-of-line)
 	    (let* ((str (buffer-substring pos (point)))
-		   (exp (save-excursion
-			  (set-buffer calc-buf)
+		   (exp (with-current-buffer calc-buf
 			  (let ((calc-user-parse-tables nil)
 				(calc-language nil)
-				(math-expr-opers math-standard-opers)
+				(math-expr-opers (math-standard-ops))
 				(calc-hashes-used 0))
 			    (math-read-expr
 			     (if (string-match ",[ \t]*\\'" str)
@@ -624,7 +626,8 @@
 			(error "Separator not allowed with { ... }?"))
 		   (if (string-match "\\`\"" sep)
 		       (setq sep (read-from-string sep)))
-		   (setq sep (calc-fix-token-name sep))
+                   (if (> (length sep) 0)
+                       (setq sep (calc-fix-token-name sep)))
 		   (setq part (nconc part
 				     (list (list sym p
 						 (and (> (length sep) 0)
@@ -688,10 +691,10 @@
            (let* ((mac (elt (nth 1 (nth 3 cmd)) 1))
                   (str (edmacro-format-keys mac t))
                   (kys (nth 3 (nth 3 cmd))))
-             (calc-edit-mode 
+             (calc-edit-mode
               (list 'calc-edit-macro-finish-edit cmdname kys)
-              t (format (concat 
-                         "Editing keyboard macro (%s, bound to %s).\n" 
+              t (format (concat
+                         "Editing keyboard macro (%s, bound to %s).\n"
                          "Original keys: %s \n")
                         cmdname kys (elt (nth 1 (nth 3 cmd)) 0)))
              (insert str "\n")
@@ -707,7 +710,7 @@
 	       (if (and defn (calc-valid-formula-func func))
 		   (let ((niceexpr (math-format-nice-expr defn (frame-width))))
 		     (calc-wrapper
-		      (calc-edit-mode 
+		      (calc-edit-mode
                        (list 'calc-finish-formula-edit (list 'quote func))
                        nil
                        (format (concat
@@ -789,7 +792,7 @@
     (when match
       (kill-line 1)
       (setq line (concat line (substring curline 0 match))))
-    (setq line (replace-regexp-in-string "SPC" " SPC " 
+    (setq line (replace-regexp-in-string "SPC" " SPC "
                   (replace-regexp-in-string " " "" line)))
     (insert line "\t\t\t")
     (if (> (current-column) 24)
@@ -814,7 +817,7 @@
       (setq line (concat line curline))
       (kill-line 1)
       (setq curline (calc-edit-macro-command)))
-    (when match 
+    (when match
       (kill-line 1)
       (setq line (concat line (substring curline 0 match))))
     (setq line (replace-regexp-in-string " " "" line))
@@ -841,7 +844,7 @@
         (setq line (concat line curline))
         (kill-line 1)
         (setq curline (calc-edit-macro-command)))
-      (when match 
+      (when match
         (kill-line 1)
         (setq line (concat line (substring curline 0 match))))
       (setq line (replace-regexp-in-string " " "" line))
@@ -1016,8 +1019,8 @@ Redefine the corresponding command."
                                         (mapcar (lambda (x) (substring x 9))
                                                 (all-completions "calcFunc-"
                                                                  obarray))
-                                        (lambda (x) 
-                                          (fboundp 
+                                        (lambda (x)
+                                          (fboundp
                                            (intern (concat "calcFunc-" x))))
                                         t)))))
                    (and (eq key ?\M-x)
@@ -1205,7 +1208,7 @@ Redefine the corresponding command."
      (calc-pop-stack 1)
      (if (math-is-true cond)
 	 (if defining-kbd-macro
-	     (message "If true.."))
+	     (message "If true..."))
        (if defining-kbd-macro
 	   (message "Condition is false; skipping to Z: or Z] ..."))
        (calc-kbd-skip-to-else-if t)))))
@@ -1282,7 +1285,7 @@ Redefine the corresponding command."
   (let* ((count 0)
 	 (parts nil)
 	 (body "")
-	 (open last-command-char)
+	 (open last-command-event)
 	 (counter initial)
 	 ch)
     (or executing-kbd-macro
@@ -2361,5 +2364,4 @@ Redefine the corresponding command."
 
 (provide 'calc-prog)
 
-;;; arch-tag: 4c5a183b-c9e5-4632-bb3f-e41a764518b0
 ;;; calc-prog.el ends here
