@@ -1,7 +1,6 @@
 ;;; tpu-edt.el --- Emacs emulating TPU emulating EDT
 
-;; Copyright (C) 1993, 1994, 1995, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 1993-1995, 2000-2012 Free Software Foundation, Inc.
 
 ;; Author: Rob Riepel <riepel@networking.stanford.edu>
 ;; Maintainer: Rob Riepel <riepel@networking.stanford.edu>
@@ -90,7 +89,7 @@
 ;;    details.
 
 ;;    Like TPU, Emacs uses multiple buffers.  Some buffers are used to hold
-;;    files you are editing; other "internal" buffers are used for Emacs' own
+;;    files you are editing; other "internal" buffers are used for Emacs's own
 ;;    purposes (like showing you help).  Here are some commands for dealing
 ;;    with buffers.
 
@@ -198,7 +197,7 @@
 
 ;;    ; Emacs uses Control-s and Control-q.  Problems can occur when using
 ;;    ; Emacs on terminals that use these codes for flow control (Xon/Xoff
-;;    ; flow control).  These lines disable Emacs' use of these characters.
+;;    ; flow control).  These lines disable Emacs's use of these characters.
 ;;    (global-unset-key "\C-s")
 ;;    (global-unset-key "\C-q")
 
@@ -460,7 +459,7 @@ Otherwise, use `spell-region'."
     ;; (define-key map "\\" nil)			; \
     (define-key map "]" 'blink-matching-open)		; ]
     (define-key map "^" 'tpu-add-at-bol)		; ^
-    (define-key map "_" 'split-window-vertically)	; -
+    (define-key map "_" 'split-window-below)	        ; -
     (define-key map "`" 'what-line)			; `
     (define-key map "a" 'tpu-toggle-newline-and-indent)	; a
     (define-key map "b" 'tpu-next-buffer)		; b
@@ -488,7 +487,7 @@ Otherwise, use `spell-region'."
     (define-key map "y" 'copy-region-as-kill)		; y
     (define-key map "z" 'suspend-emacs)			; z
     ;; (define-key map "{" nil)				; {
-    (define-key map "|" 'split-window-horizontally)	; |
+    (define-key map "|" 'split-window-right)	        ; |
     ;; (define-key map "}" nil)				; }
     (define-key map "~" 'exchange-point-and-mark)	; ~
     (define-key map "\177" 'delete-window)		; <X]
@@ -884,7 +883,7 @@ With argument, fill and justify."
          (setq tpu-newline-and-indent-p t)
          (tpu-local-set-key "\C-m" 'newline-and-indent)))
   (tpu-update-mode-line)
-  (and (interactive-p)
+  (and (called-interactively-p 'interactive)
        (message "Carriage return inserts a newline%s"
 		(if tpu-newline-and-indent-p " and indents." "."))))
 
@@ -980,7 +979,10 @@ and the total number of lines in the buffer."
 ;;;
 ;;;###autoload
 (define-minor-mode tpu-edt-mode
-  "TPU/edt emulation."
+  "Toggle TPU/edt emulation on or off.
+With a prefix argument ARG, enable the mode if ARG is positive,
+and disable it otherwise.  If called from Lisp, enable the mode
+if ARG is omitted or nil."
   :global t
   (if tpu-edt-mode (tpu-edt-on) (tpu-edt-off)))
 
@@ -1279,9 +1281,9 @@ kill modified buffers without asking."
 
 (defun tpu-make-file-buffer-list (buffer-list)
   "Return names from BUFFER-LIST excluding those beginning with a space or star."
-  (delq nil (mapcar '(lambda (b)
-                       (if (or (= (aref (buffer-name b) 0) ? )
-                               (= (aref (buffer-name b) 0) ?*)) nil b))
+  (delq nil (mapcar (lambda (b)
+                      (if (or (= (aref (buffer-name b) 0) ?\s)
+                              (= (aref (buffer-name b) 0) ?*)) nil b))
                     buffer-list)))
 
 (defun tpu-next-window nil
@@ -1305,7 +1307,7 @@ kill modified buffers without asking."
   (interactive)
   (setq tpu-regexp-p (not tpu-regexp-p))
   (tpu-set-search)
-  (and (interactive-p)
+  (and (called-interactively-p 'interactive)
        (message "Regular expression search and substitute %sabled."
 		(if tpu-regexp-p "en" "dis"))))
 
@@ -1450,7 +1452,7 @@ Used for reversing a search in progress."
   (interactive)
   (setq tpu-searching-forward (not tpu-searching-forward))
   (tpu-set-search t)
-  (and (interactive-p)
+  (and (called-interactively-p 'interactive)
        (message "Searching %sward."
 		(if tpu-searching-forward "for" "back"))))
 
@@ -1501,7 +1503,7 @@ Used for reversing a search in progress."
   (setq tpu-rectangular-p (not tpu-rectangular-p))
   (setq tpu-rectangle-string (if tpu-rectangular-p " Rect" ""))
   (tpu-update-mode-line)
-  (and (interactive-p)
+  (and (called-interactively-p 'interactive)
        (message "Rectangular cut and paste %sabled."
 		(if tpu-rectangular-p "en" "dis"))))
 
@@ -2273,7 +2275,7 @@ Accepts a prefix argument for the number of tpu-pan-columns to scroll."
   "Toggle control key bindings between TPU-edt and Emacs."
   (interactive)
   (tpu-reset-control-keys (not tpu-control-keys))
-  (and (interactive-p)
+  (and (called-interactively-p 'interactive)
        (message "Control keys function with %s bindings."
 		(if tpu-control-keys "TPU-edt" "Emacs"))))
 
@@ -2374,7 +2376,7 @@ If FILE is nil, try to load a default file.  The default file names are
     (and (tpu-y-or-n-p "Copy key definitions to the new file now? ")
 	 (condition-case conditions
              (copy-file oldname newname)
-	   (tpu-error (message "Sorry, couldn't copy - %s." (cdr conditions)))))
+	   (error (message "Sorry, couldn't copy - %s." (cdr conditions)))))
     (kill-buffer "*TPU-Notice*")))
 
 (defvar tpu-edt-old-global-values nil)
@@ -2438,11 +2440,14 @@ If FILE is nil, try to load a default file.  The default file names are
 
 
 ;;;### (autoloads (tpu-set-cursor-bound tpu-set-cursor-free tpu-set-scroll-margins
-;;;;;;  tpu-cursor-free-mode) "tpu-extras" "tpu-extras.el" "a7e10e36bed23a02f05ec5a69ae8569a")
+;;;;;;  tpu-cursor-free-mode) "tpu-extras" "tpu-extras.el" "76f06905db4c5bfb3b86491a51512a0e")
 ;;; Generated autoloads from tpu-extras.el
 
 (autoload 'tpu-cursor-free-mode "tpu-extras" "\
 Minor mode to allow the cursor to move freely about the screen.
+With a prefix argument ARG, enable the mode if ARG is positive,
+and disable it otherwise.  If called from Lisp, enable the mode
+if ARG is omitted or nil.
 
 \(fn &optional ARG)" t nil)
 
@@ -2465,5 +2470,4 @@ Constrain the cursor to the flow of the text.
 
 (provide 'tpu-edt)
 
-;; arch-tag: f3dfe61c-2cbd-4f73-b9cc-eb215020b857
 ;;; tpu-edt.el ends here
