@@ -1,7 +1,7 @@
 /* Interface of a lightweight menubar widget.
+
+Copyright (C) 2002-2012  Free Software Foundation, Inc.
 Copyright (C) 1992 Lucid, Inc.
-Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008
-  Free Software Foundation, Inc.
 
 This file is part of the Lucid Widget Library.
 
@@ -77,7 +77,8 @@ typedef struct _XlwMenuClassRec *XlwMenuWidgetClass;
 
 extern WidgetClass xlwMenuWidgetClass;
 
+extern int xlwmenu_window_p (Widget w, Window window);
+extern void xlwmenu_redisplay (Widget);
+
 #endif /* _XlwMenu_h */
 
-/* arch-tag: 0c019735-d61b-4080-be85-4fdd6e50ae07
-   (do not change this comment) */
